package io.tswf.minecraft.blockventure.core.inventory.screenhandler

import groovy.transform.CompileStatic
import groovy.transform.NamedVariant
import io.tswf.groovy.bettergroovy.transforms.namedvariantdefaults.NamedVariantDefaults
import io.tswf.groovy.bettergroovy.transforms.nullsafe.NullSafe
import io.tswf.groovy.bettergroovy.transforms.validation.checkedmapconstructor.CheckedMapConstructor
import io.tswf.minecraft.blockventure.core.api.gui.handler.IScreenHandlerMultipleSlotsQuickMoveHelper
import io.tswf.minecraft.blockventure.core.api.gui.handler.IScreenHandlerStackInSlotQuickMoveHelper
import it.unimi.dsi.fastutil.ints.IntOpenHashSet
import it.unimi.dsi.fastutil.ints.IntSet
import net.minecraft.entity.player.PlayerEntity
import net.minecraft.item.ItemStack
import net.minecraft.screen.ScreenHandler

@CompileStatic
class MultipleSlotsQuickMoveHelper implements IScreenHandlerMultipleSlotsQuickMoveHelper {

    Map<String, SlotsGroup> slotsToQuickMoveHelpers = new HashMap<>()
    ScreenHandler screenHandler

    MultipleSlotsQuickMoveHelper(@NullSafe ScreenHandler screenHandler) {
        this.screenHandler = screenHandler
    }

    ItemStack quickMoveStack(PlayerEntity player, int slot) {
        def result = null
        for (def slotsGroup : slotsToQuickMoveHelpers.values()) {
            if (slotsGroup.slotIdsSet.contains(slot)) {
                result = slotsGroup.quickMoveHelper.quickMoveStack(player, slot)
                if (result == null || result == ItemStack.EMPTY) {
                    break
                }
            }
        }

        return result ?: ItemStack.EMPTY
    }

    ISlotsGroup getGroup(String name) {
        return slotsToQuickMoveHelpers.computeIfAbsent(name) { new SlotsGroup(name, screenHandler) }
    }

    ISlotsGroup getAt(String name) {
        return getGroup(name)
    }

    @CompileStatic
    @CheckedMapConstructor
    static class SlotsGroup implements ISlotsGroup {
        String name
        IScreenHandlerStackInSlotQuickMoveHelper quickMoveHelper
        ScreenHandler screenHandler
        IntSet slotIdsSet = new IntOpenHashSet()

        SlotsGroup(String name, ScreenHandler screenHandler) {
            this.quickMoveHelper = new StackInSlotQuickMoveHelper(screenHandler)
            this.name = name
            this.screenHandler = screenHandler
        }

        void addTargetSlot(int slotIndex) {
            quickMoveHelper.addSlotToTransfer(slotIndex)
        }

        void addTargetSlots(Iterable<Integer> slotIndexes) {
            for (def slotIndex : slotIndexes) {
                addTargetSlot(slotIndex)
            }
        }

        @NamedVariantDefaults
        @NamedVariant
        void addTargetSlots(ISlotsGroup group, boolean sort = true, boolean reverse = false) {
            def slots = group.sourceSlots.toList()

            if (sort) {
                slots = slots.toSorted()
            }

            if (reverse) {
                slots = slots.toList()
                Collections.reverse(slots)
            }

            addTargetSlots(slots)
        }

        void addSourceSlot(int slotIndex) {
            slotIdsSet.add(slotIndex)
        }

        void addSourceSlots(Iterable<Integer> slotIndexes) {
            for (def slotIndex : slotIndexes) {
                addSourceSlot(slotIndex)
            }
        }

        void addSourceSlots(Runnable run) {
            def prevSlots = screenHandler.slots.toList()
            run.run()
            def newSlots = screenHandler.slots.stream()
                .filter {
                    it !in prevSlots
                }
                .map {
                    screenHandler.slots.indexOf(it)
                }
                .filter {
                    it != null && it >= 0
                }
                .toList()

            addSourceSlots(newSlots)
        }

        @Override
        List<Integer> getSourceSlots() {
            return slotIdsSet.toList()
        }
    }
}
