package io.tswf.minecraft.blockventure.core.inventory.screenhandler

import groovy.transform.CompileStatic
import io.tswf.groovy.bettergroovy.transforms.nullsafe.NullSafe
import io.tswf.minecraft.blockventure.core.api.gui.handler.IScreenHandlerStackInSlotQuickMoveHelper
import io.tswf.minecraft.blockventure.core.transmutation.view.ViewScreenHandler
import it.unimi.dsi.fastutil.ints.IntArrayList
import it.unimi.dsi.fastutil.ints.IntList
import net.minecraft.entity.player.PlayerEntity
import net.minecraft.item.ItemStack
import net.minecraft.screen.ScreenHandler
import org.antlr.runtime.misc.IntArray

@CompileStatic
class StackInSlotQuickMoveHelper implements IScreenHandlerStackInSlotQuickMoveHelper {
    
    IntList slotsToTransferIndexes = new IntArrayList()
    ScreenHandler screenHandler

    StackInSlotQuickMoveHelper(
            @NullSafe
            ScreenHandler screenHandler,

            @NullSafe
            Iterable<Integer> slotsToTransferIndexes
    ) {
        this(screenHandler, convertIterableToArray(slotsToTransferIndexes))
    }

    StackInSlotQuickMoveHelper(
            @NullSafe
            ScreenHandler screenHandler,

            @NullSafe
            int... slotsToTransferIndexes = new int[0]
    ) {
        this.screenHandler = screenHandler
        for (int slotToTransfer : slotsToTransferIndexes) {
            this.slotsToTransferIndexes.add(slotToTransfer)
        }
    }

    @Override
    ItemStack quickMoveStack(@NullSafe PlayerEntity player, int slotIndex) {
        def viewScreenHandler = (ViewScreenHandler) screenHandler
        def newStackInFromTransferSlot = ItemStack.EMPTY
        def slotFromTransfer = screenHandler.slots.get(slotIndex)

        if (slotFromTransfer != null && slotFromTransfer.hasStack()) {
            def stackInFromTransferSlot = slotFromTransfer.stack
            newStackInFromTransferSlot = stackInFromTransferSlot.copy()

            for (def slotToTransferIndex : slotsToTransferIndexes) {
                def isStackTransferredFully = viewScreenHandler.callInsertItem(stackInFromTransferSlot, slotToTransferIndex, slotToTransferIndex + 1, false)

                if (isStackTransferredFully) {
                    break
                }
            }

            if (stackInFromTransferSlot.isEmpty()) {
                slotFromTransfer.stack = ItemStack.EMPTY
            } else {
                slotFromTransfer.markDirty()
            }

            slotFromTransfer.onTakeItem(player, stackInFromTransferSlot)

            if (stackInFromTransferSlot.count == newStackInFromTransferSlot.count) {
                return ItemStack.EMPTY
            }
        }

        return newStackInFromTransferSlot
    }

    @Override
    void addSlotToTransfer(int slotIndex) {
        this.slotsToTransferIndexes.add(slotIndex)
    }

    protected static int[] convertIterableToArray(Iterable<Integer> iter) {
        return iter.stream()
            .mapToInt { it }
            .toArray()
    }
}
